var margin = {top: 20, right: 250, bottom: 250, left: 55},
    width = 960 - margin.left - margin.right,
    height = 500 - margin.top - margin.bottom;

var xScale = d3.scale.ordinal()
    .rangeRoundBands([0, width], .3);

var yScale = d3.scale.linear()
    .rangeRound([height, 0]);

var color = d3.scale.category20();

var xAxis = d3.svg.axis()
    .scale(xScale)
    .orient("bottom")
    .innerTickSize([0]);

var yAxis = d3.svg.axis()
    .scale(yScale)
    .orient("left")
    .tickFormat(d3.format(".2s")); // for the stacked totals version

var stack = d3.layout
    .stack();

var svg = d3.select("svg")
    .attr("width", width + margin.left + margin.right)
    .attr("height", height + margin.top + margin.bottom)
  .append("g")
    .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

d3.csv("data3.csv", function(error, data) {

  if (error) {
    console.log(error);
  }

  data.sort(function(a,b) { return a.Province - b.Province;});
  // how would we sort by largest total bar?  what would we have to calculate?

  //var ageRange = ["0-27 days","1-59 months","5-14 years","15-29 years","30-49 years","50-59 years","60-69 years","70+ years"];
  
  var ageRange = ["Travel","Relative","Social","Unknown"];

  var dataToStack = makeData(ageRange, data);
  console.log(dataToStack);
  var stacked = stack(dataToStack);
  console.log(stacked);

  xScale.domain(data.map(function(d) { return d.Province; }));

  svg.append("g")
      .attr("class", "x axis")
      .attr("transform", "translate(0," + height + ")")
      .call(xAxis)
      .selectAll("text")
        .attr("dy", ".5em")
        .attr("transform", "rotate(-30)")
        .style("text-anchor", "end");

  svg.append("g")
      .attr("class", "y axis")
      .call(yAxis)
    .append("text")
      .attr("transform", "rotate(-90)")
      .attr("y", -margin.left)
      .attr("x", -(height-margin.bottom)/2)
      .attr("dy", ".71em")
      .style("text-anchor", "end")
      .text("Case Number");

  var cause = svg.selectAll(".cause")
      .data(stacked)
    .enter().append("g")
      .attr("class", "cause")
      .style("fill", function(d, i) { return color(i); });

  var rectangles = cause.selectAll("rect")
      .data(function(d) {
        console.log("array for a rectangle");
        return d; })  // this just gets the array for bar segment.
    .enter().append("rect")
        .attr("width", xScale.rangeBand());

    // this just draws them in the default way, now they're appended.
  transitionCount();

  drawLegend();

  var myTooltip = d3.select("body")
    .append("div")
    .attr("class", "myTooltip");

var percentClicked = false;

  d3.selectAll("input").on("change", handleFormClick);

  // All the functions for stuff above!

  function handleFormClick() {
    if (this.value === "bypercent") {
      percentClicked = true;
      transitionPercent();
    } else {
      percentClicked = false;
      transitionCount();
    }
  }

  function makeData (age,data) {
    return ageRange.map(function(age) {
        return data.map(function(d) {
          return {x: d["Province"], y: +d[age], ageRange: age};
        })
      });
  }

  function transitionPercent() {
    yAxis.tickFormat(d3.format("%"));
    stack.offset("expand");  // use this to get it to be relative/normalized!
    var stacked = stack(makeData(ageRange, data));
    // call function to do the bars, which is same across both formats.
    transitionRects(stacked);
  }

  function transitionCount() {
    yAxis.tickFormat(d3.format(".2s")); // for the stacked totals version
    stack.offset("zero");
    var stacked = stack(makeData(ageRange, data));
    transitionRects(stacked);
    console.log("stacked", stacked);
    }

  function transitionRects(stacked) {

    // this domain is using the last of the stacked arrays, which is the last illness, and getting the max height.
    yScale.domain([0, d3.max(stacked[stacked.length-1], function(d) { return d.y0 + d.y; })]);

    // attach new fixed data
    var cause = svg.selectAll(".cause")
      .data(stacked);

    // same on the rects
    cause.selectAll("rect")
      .data(function(d) {
        console.log("array for a rectangle");
        return d;
      })  // this just gets the array for bar segment.

    svg.selectAll("g.cause rect")
      .transition()
      .duration(250)
      .attr("x", function(d) {
        return xScale(d.x); })
      .attr("y", function(d) {
        return yScale(d.y0 + d.y); }) //
      .attr("height", function(d) {
        return yScale(d.y0) - yScale(d.y0 + d.y); });  // height is base - tallness

    svg.selectAll(".y.axis").transition().call(yAxis);
  }

  // Building a legend by hand, based on http://bl.ocks.org/mbostock/3886208
  function drawLegend() {

    var legend = svg.selectAll(".legend")
        .data(color.domain().slice()) // what do you think this does?
      .enter().append("g")
        .attr("class", "legend")
        .attr("transform", function(d, i) { return "translate(0," + i * 20 + ")"; });

    legend.append("rect")
        .attr("x", width)
        .attr("width", 18)
        .attr("height", 18)
        .style("fill", color);

    legend.append("text")
        .attr("x", width + 24)
        .attr("y", 9)
        .attr("dy", ".35em")
        .style("text-anchor", "start")
        .text(function(d, i) { return ageRange[i]; });
  }

   rectangles
        .on("mouseover", mouseoverFunc)
        .on("mousemove", mousemoveFunc)
        .on("mouseout", mouseoutFunc);


    function mouseoverFunc(d) {
      console.log("moused over", d.x);
        if(percentClicked) {
            myTooltip
            .style("display", null)
            .html("<p><span class='tooltipHeader'>" + d.x + "</span><br>"+ d.ageRange + ": " + d3.format("%")(d.y) + "</p>");
        } else {
                         console.log("method", d.ageRange, "percent", d.y);
        myTooltip
            .style("display", null)
            .html("<p><span class='tooltipHeader'>" + d.x + "</span><br>"+ d.ageRange + ": " +d.y + "</p>");
        }
    }

    function mousemoveFunc(d) {
        myTooltip
            .style("top", (d3.event.pageY - 5) + "px")
            .style("left", (d3.event.pageX + 10) + "px");
    }
    
    function mouseoutFunc(d) {
        return myTooltip.style("display", "none"); // this sets it to invisible!
    }
});